package gcewing.lighting.ic2;

import net.minecraft.block.*;
import net.minecraft.nbt.*;
import net.minecraft.tileentity.*;
import net.minecraftforge.common.*;
import net.minecraftforge.common.util.*;

import ic2.api.*;
import ic2.api.energy.*;
import ic2.api.energy.event.*;
import ic2.api.energy.tile.*;

import gcewing.lighting.*;

public class TEFloodlightIC2 extends TileEntity implements IEnergySink {

	final double maxEnergy = 64;
	final int maxInput = 32;
	final double energyUsedPerTick = 1;
	
	double energy = 0;
	boolean addedToEnergyNet = false;
	
	@Override
	public void readFromNBT(NBTTagCompound nbt) {
		super.readFromNBT(nbt);
		energy = nbt.getDouble("energy");
	}
	
	@Override
	public void writeToNBT(NBTTagCompound nbt) {
		super.writeToNBT(nbt);
		nbt.setDouble("energy", energy);
	}
	
	public int getMaxSafeInput() {
		return maxInput;
	}

	@Override
	public int getSinkTier() {
		return 1; // LV
	}
	
	@Override
	public boolean acceptsEnergyFrom(TileEntity emitter, ForgeDirection direction) {
		//System.out.printf("TEFloodlightIC2.acceptsEnergyFrom\n");
		return true;
	}
	
	@Override
//	public double demandedEnergyUnits() {
	public double getDemandedEnergy() {
		double req;
    if (energy < maxEnergy / 2)
    	req = maxEnergy - energy;
    else
    	req = 0;
		//System.out.printf("TEFloodlightIC2.demandsEnergy: %s\n", req);
		return req;
  }
  
  @Override
//  public double injectEnergyUnits(ForgeDirection directionFrom, double amount) {
  public double injectEnergy(ForgeDirection directionFrom, double amount, double voltage) {
  	//System.out.printf("TEFloodlightIC2.injectEnergyUnits: %s\n", amount);
  	if (amount > getMaxSafeInput()) {
  		GregsLightingIC2.explodeMachineAt(getWorldObj(), xCoord, yCoord, zCoord);
  		return 0;
  	}
  	boolean hadEnergy = energy > 0;
  	energy += amount;
  	if (hadEnergy != (energy > 0)) {
  		markDirty();
  		updateBlock();
  	}
  	return 0;
  }

	public void updateEntity() {
		if (!worldObj.isRemote) {
			if (!this.addedToEnergyNet) {
				MinecraftForge.EVENT_BUS.post(new EnergyTileLoadEvent(this));
				this.addedToEnergyNet = true;
			}
			if (isActive()) {
				//System.out.printf("TEFloodlightIC2.updateEntity: using %d energy from %d\n",
				//	energyUsedPerTick, energy);
				energy -= energyUsedPerTick;
				if (energy < 0)
					energy = 0;
				markDirty();
				if (!isActive())
					updateBlock();
			}
		}
	}
	
	@Override
	public void invalidate() {
		removeFromEnergyNet();
		super.invalidate();
	}
	
	@Override
	public void onChunkUnload() {
		removeFromEnergyNet();
		super.onChunkUnload();
	}
	
	void removeFromEnergyNet() {
		if (addedToEnergyNet) {
			MinecraftForge.EVENT_BUS.post(new EnergyTileUnloadEvent(this));
			addedToEnergyNet = false;
		}
	}
	
	BlockFloodlightIC2 getBlock() {
		return (BlockFloodlightIC2)getBlockType();
	}

	void updateBlock() {
		BlockFloodlightIC2 block = getBlock();
		block.update(worldObj, xCoord, yCoord, zCoord);
		block.updateBeam(worldObj, xCoord, yCoord, zCoord);
	}
	
	public boolean isActive() {
		boolean on = switchedOn();
		boolean redstone =  receivingRedstoneSignal();
		//System.out.printf("TEFloodlightIC2.isActive: %s at (%d, %d, %d) energy = %d, on = %s, redstone = %s\n",
		//	this, xCoord, yCoord, zCoord, energy, on, redstone);
		return energy > 0 && (on || redstone);
	}
	
	public boolean switchedOn() {
		BlockFloodlightIC2 block = getBlock();
		boolean result = block.switchedOn;
		//System.out.printf("TEFloodlightIC2.switchedOn: id = %s switchedOn = %s\n",
		//	block.blockID, result);
		return result;
	}
	
	boolean receivingRedstoneSignal() {
		return worldObj.isBlockIndirectlyGettingPowered(xCoord, yCoord, zCoord);
	}
	
}
